@@ -39,8 +39,9 @@ FileTransferJob::FileTransferJob(const QSharedPointer<QIODevice>& origin, qint64
     , mWritten(0)
 {
     Q_ASSERT(mOrigin);
+    Q_ASSERT(mOrigin->isReadable());
     if (mDestination.scheme().isEmpty()) {
-        qWarning() << "Destination QUrl" << mDestination << "lacks a scheme. Setting its scheme to 'file'.";
+        qCWarning(KDECONNECT_CORE) << "Destination QUrl" << mDestination << "lacks a scheme. Setting its scheme to 'file'.";
         mDestination.setScheme("file");
     }
 
@@ -61,16 +62,19 @@ void FileTransferJob::start()
 void FileTransferJob::doStart()
 {
     description(this, i18n("Receiving file over KDE Connect"),
-        QPair<QString, QString>(i18nc("File transfer origin", "From"), mFrom)
+        { i18nc("File transfer origin", "From"), mFrom }
     );
 
     if (mDestination.isLocalFile() && QFile::exists(mDestination.toLocalFile())) {
         setError(2);
         setErrorText(i18n("Filename already present"));
         emitResult();
+        return;
     }
 
-    startTransfer();
+    if (mOrigin->bytesAvailable())
+        startTransfer();
+    connect(mOrigin.data(), &QIODevice::readyRead, this, &FileTransferJob::startTransfer);
 }
 
 void FileTransferJob::startTransfer()
@@ -78,8 +82,8 @@ void FileTransferJob::startTransfer()
     setProcessedAmount(Bytes, 0);
     mTime = QTime::currentTime();
     description(this, i18n("Receiving file over KDE Connect"),
-                        QPair<QString, QString>(i18nc("File transfer origin", "From"), mFrom),
-                        QPair<QString, QString>(i18nc("File transfer destination", "To"), mDestination.toLocalFile()));
+                        { i18nc("File transfer origin", "From"), mFrom },
+                        { i18nc("File transfer destination", "To"), mDestination.toLocalFile() });
 
     QNetworkRequest req(mDestination);
     req.setHeader(QNetworkRequest::ContentLengthHeader, totalAmount(Bytes));
@@ -89,19 +93,25 @@ void FileTransferJob::startTransfer()
         setProcessedAmount(Bytes, bytesSent);
         emitSpeed(bytesSent/mTime.elapsed());
     });
+    connect(mReply, static_cast<void (QNetworkReply::*)(QNetworkReply::NetworkError)>(&QNetworkReply::error),
+            this, &FileTransferJob::transferFailed);
     connect(mReply, &QNetworkReply::finished, this, &FileTransferJob::transferFinished);
 }
 
+void FileTransferJob::transferFailed(QNetworkReply::NetworkError error)
+{
+    qCDebug(KDECONNECT_CORE) << "Couldn't transfer the file successfully" << error << mReply->errorString();
+    setError(error);
+    setErrorText(i18n("Received incomplete file: %1", mReply->errorString()));
+    emitResult();
+
+    mReply->close();
+}
+
 void FileTransferJob::transferFinished()
 {
     //TODO: MD5-check the file
-    if (mReply->error()) {
-        qCDebug(KDECONNECT_CORE) << "Couldn't transfer the file successfully" << mReply->errorString();
-        setError(mReply->error());
-        setErrorText(i18n("Received incomplete file: %1", mReply->errorString()));
-    } else {
-        qCDebug(KDECONNECT_CORE) << "Finished transfer" << mDestination;
-    }
+    qCDebug(KDECONNECT_CORE) << "Finished transfer" << mDestination;
 
     emitResult();
 }
