@@ -22,16 +22,15 @@
 #ifndef FILETRANSFERJOB_H
 #define FILETRANSFERJOB_H
 
+#include <KJob>
+
 #include <QIODevice>
 #include <QTime>
-#include <QTemporaryFile>
 #include <QSharedPointer>
-
-#include <KJob>
 #include <QUrl>
-#include <QNetworkAccessManager>
 #include <QNetworkReply>
-#include <QBuffer>
+
+#include "kdeconnectcore_export.h"
 
 /**
  * @short It will stream a device into a url destination
@@ -39,7 +38,7 @@
  * Given a QIODevice, the file transfer job will use the system's QNetworkAccessManager
  * for putting the stream into the requested location.
  */
-class FileTransferJob
+class KDECONNECTCORE_EXPORT FileTransferJob
     : public KJob
 {
     Q_OBJECT
@@ -51,7 +50,7 @@ public:
      * @p destination specifies where these contents should be stored
      */
     FileTransferJob(const QSharedPointer<QIODevice>& origin, qint64 size, const QUrl &destination);
-    virtual void start() Q_DECL_OVERRIDE;
+    void start() override;
     QUrl destination() const { return mDestination; }
     void setOriginName(QString from) { mFrom = from; }
 
@@ -59,10 +58,11 @@ private Q_SLOTS:
     void doStart();
 
 protected:
-    bool doKill() Q_DECL_OVERRIDE;
+    bool doKill() override;
 
 private:
     void startTransfer();
+    void transferFailed(QNetworkReply::NetworkError error);
     void transferFinished();
 
     QSharedPointer<QIODevice> mOrigin;
