@@ -44,6 +44,8 @@
 
 K_PLUGIN_FACTORY(KdeConnectKcmFactory, registerPlugin<KdeConnectKcm>();)
 
+static QString createId() { return QStringLiteral("kcm")+QString::number(QCoreApplication::applicationPid()); }
+
 KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
     : KCModule(KAboutData::pluginData("kdeconnect-kcm"), parent)
     , kcmUi(new Ui::KdeConnectKcmUi())
@@ -83,7 +85,7 @@ KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
     kcmUi->rename_edit->setText(daemon->announcedName());
     setRenameMode(false);
 
-    setButtons(KCModule::NoAdditionalButton);
+    setButtons(KCModule::Help | KCModule::NoAdditionalButton);
 
     connect(devicesModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(resetSelection()));
@@ -104,6 +106,7 @@ KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
     connect(kcmUi->renameShow_button,SIGNAL(clicked()),
             this, SLOT(renameShow()));
 
+    daemon->acquireDiscoveryMode(createId());
 }
 
 void KdeConnectKcm::renameShow()
@@ -133,11 +136,13 @@ void KdeConnectKcm::setRenameMode(bool b) {
 
 KdeConnectKcm::~KdeConnectKcm()
 {
+    daemon->releaseDiscoveryMode(createId());
     delete kcmUi;
 }
 
 void KdeConnectKcm::refresh()
 {
+    daemon->acquireDiscoveryMode(createId());
     daemon->forceOnNetworkChange();
 }
 
@@ -151,14 +156,8 @@ void KdeConnectKcm::resetSelection()
 
 void KdeConnectKcm::deviceSelected(const QModelIndex& current)
 {
-
-    kcmUi->noDevicePlaceholder->setVisible(false);
-
     if (currentDevice) {
-        disconnect(currentDevice,SIGNAL(pairingChanged(bool)),
-            this, SLOT(pairingChanged(bool)));
-        disconnect(currentDevice,SIGNAL(pairingFailed(QString)),
-            this, SLOT(pairingFailed(QString)));
+        disconnect(currentDevice, 0, this, 0);
     }
 
     //Store previous device config
@@ -173,6 +172,7 @@ void KdeConnectKcm::deviceSelected(const QModelIndex& current)
     currentIndex = sortProxyModel->mapToSource(current);
     currentDevice = devicesModel->getDevice(currentIndex.row());
 
+    kcmUi->noDevicePlaceholder->setVisible(false);
     bool valid = (currentDevice != nullptr && currentDevice->isValid());
     kcmUi->deviceInfo->setVisible(valid);
     if (!valid) {
@@ -180,24 +180,34 @@ void KdeConnectKcm::deviceSelected(const QModelIndex& current)
     }
 
     kcmUi->messages->setVisible(false);
-    if (currentDevice->pairRequested()) {
-        kcmUi->progressBar->setVisible(true);
-        kcmUi->unpair_button->setVisible(false);
+    kcmUi->progressBar->setVisible(false);
+    if (currentDevice->isTrusted()) {
+        kcmUi->unpair_button->setVisible(true);
         kcmUi->pair_button->setVisible(false);
-        kcmUi->ping_button->setVisible(false);
+        kcmUi->ping_button->setVisible(true);
     } else {
-        kcmUi->progressBar->setVisible(false);
-        if (currentDevice->isPaired()) {
-            kcmUi->unpair_button->setVisible(true);
-            kcmUi->pair_button->setVisible(false);
-            kcmUi->ping_button->setVisible(true);
-        } else {
-            kcmUi->unpair_button->setVisible(false);
-            kcmUi->pair_button->setVisible(true);
-            kcmUi->ping_button->setVisible(false);
-        }
+        kcmUi->unpair_button->setVisible(false);
+        kcmUi->pair_button->setVisible(true);
+        kcmUi->ping_button->setVisible(false);
     }
+    resetDeviceView();
 
+    connect(currentDevice, SIGNAL(pluginsChanged()), this, SLOT(resetCurrentDevice()));
+    connect(currentDevice, SIGNAL(trustedChanged(bool)), this, SLOT(trustedChanged(bool)));
+    connect(currentDevice, SIGNAL(pairingError(QString)), this, SLOT(pairingFailed(QString)));
+}
+
+void KdeConnectKcm::resetCurrentDevice()
+{
+    const QStringList supportedPluginNames = currentDevice->supportedPlugins();
+
+    if (m_oldSupportedPluginNames != supportedPluginNames) {
+        resetDeviceView();
+    }
+}
+
+void KdeConnectKcm::resetDeviceView()
+{
     //KPluginSelector has no way to remove a list of plugins and load another, so we need to destroy and recreate it each time
     delete kcmUi->pluginSelector;
     kcmUi->pluginSelector = new KPluginSelector(this);
@@ -206,19 +216,25 @@ void KdeConnectKcm::deviceSelected(const QModelIndex& current)
     kcmUi->pluginSelector->setConfigurationArguments(QStringList(currentDevice->id()));
 
     kcmUi->name_label->setText(currentDevice->name());
-    kcmUi->status_label->setText(currentDevice->isPaired()? i18n("(paired)") : i18n("(unpaired)"));
-
-    connect(currentDevice,SIGNAL(pairingChanged(bool)),
-            this, SLOT(pairingChanged(bool)));
-    connect(currentDevice,SIGNAL(pairingFailed(QString)),
-            this, SLOT(pairingFailed(QString)));
+    kcmUi->status_label->setText(currentDevice->isTrusted()? i18n("(trusted)") : i18n("(not trusted)"));
 
     const QList<KPluginInfo> pluginInfo = KPluginInfo::fromMetaData(KPluginLoader::findPlugins("kdeconnect/"));
-    KSharedConfigPtr deviceConfig = KSharedConfig::openConfig(currentDevice->pluginsConfigFile());
-    kcmUi->pluginSelector->addPlugins(pluginInfo, KPluginSelector::ReadConfigFile, i18n("Plugins"), QString(), deviceConfig);
+    QList<KPluginInfo> availablePluginInfo;
+    QList<KPluginInfo> unsupportedPluginInfo;
 
-    connect(kcmUi->pluginSelector, SIGNAL(changed(bool)),
-            this, SLOT(pluginsConfigChanged()));
+    m_oldSupportedPluginNames = currentDevice->supportedPlugins();
+    for (auto it = pluginInfo.cbegin(), itEnd = pluginInfo.cend(); it!=itEnd; ++it) {
+        if (m_oldSupportedPluginNames.contains(it->pluginName())) {
+            availablePluginInfo.append(*it);
+        } else {
+            unsupportedPluginInfo.append(*it);
+        }
+    }
+
+    KSharedConfigPtr deviceConfig = KSharedConfig::openConfig(currentDevice->pluginsConfigFile());
+    kcmUi->pluginSelector->addPlugins(availablePluginInfo, KPluginSelector::ReadConfigFile, i18n("Available plugins"), QString(), deviceConfig);
+    kcmUi->pluginSelector->addPlugins(unsupportedPluginInfo, KPluginSelector::ReadConfigFile, i18n("Unavailable plugins"), QString(), deviceConfig);
+    connect(kcmUi->pluginSelector, SIGNAL(changed(bool)), this, SLOT(pluginsConfigChanged()));
 
 }
 
@@ -250,20 +266,20 @@ void KdeConnectKcm::pairingFailed(const QString& error)
 {
     if (sender() != currentDevice) return;
 
-    pairingChanged(false);
+    trustedChanged(false);
 
     kcmUi->messages->setText(i18n("Error trying to pair: %1",error));
     kcmUi->messages->animatedShow();
 }
 
-void KdeConnectKcm::pairingChanged(bool paired)
+void KdeConnectKcm::trustedChanged(bool paired)
 {
     DeviceDbusInterface* senderDevice = (DeviceDbusInterface*) sender();
     if (senderDevice != currentDevice) return;
 
     kcmUi->pair_button->setVisible(!paired);
     kcmUi->unpair_button->setVisible(paired);
-    kcmUi->progressBar->setVisible(senderDevice->pairRequested());
+    kcmUi->progressBar->setVisible(false);
     kcmUi->ping_button->setVisible(paired);
     kcmUi->status_label->setText(paired ? i18n("(paired)") : i18n("(unpaired)"));
 }
