@@ -29,8 +29,8 @@ class KdeConnectDeclarativePlugin : public QQmlExtensionPlugin
 
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
 
-    virtual void registerTypes(const char* uri) override;
-    virtual void initializeEngine(QQmlEngine *engine, const char *uri) override;
+    void registerTypes(const char* uri) override;
+    void initializeEngine(QQmlEngine *engine, const char *uri) override;
 };
 
 #endif // KDECONNECTDECLARATIVEPLUGIN_H
