@@ -36,8 +36,8 @@ public:
     explicit PauseMusicPlugin(QObject *parent, const QVariantList &args);
 
 public Q_SLOTS:
-    virtual bool receivePackage(const NetworkPackage& np);
-    virtual void connected() { };
+    bool receivePackage(const NetworkPackage& np) override;
+    void connected() override { }
 
     /**
      * @returns 0 if not muted, 1 if muted or -1 if there was an error
@@ -46,6 +46,7 @@ public Q_SLOTS:
     
 private:
     QSet<QString> pausedSources;
+    bool muted;
 
 };
 
